@@ -29,8 +29,10 @@ from .solvers import (
     dense_solvers,
     ecos_solve_qp,
     gurobi_solve_qp,
+    highs_solve_qp,
     mosek_solve_qp,
     osqp_solve_qp,
+    proxqp_solve_qp,
     qpoases_solve_qp,
     qpswift_solve_qp,
     quadprog_solve_qp,
@@ -39,7 +41,7 @@ from .solvers import (
 )
 from .utils import print_matrix_vector
 
-__version__ = "2.2.0"
+__version__ = "2.5.0"
 
 __all__ = [
     "__version__",
@@ -49,9 +51,11 @@ __all__ = [
     "dense_solvers",
     "ecos_solve_qp",
     "gurobi_solve_qp",
+    "highs_solve_qp",
     "mosek_solve_qp",
     "osqp_solve_qp",
     "print_matrix_vector",
+    "proxqp_solve_qp",
     "qpoases_solve_qp",
     "qpswift_solve_qp",
     "quadprog_solve_qp",
