@@ -20,20 +20,21 @@
 
 """Solver interface for CVXOPT."""
 
-from typing import Optional
+from typing import Any, Dict, Optional, Union
 
-from cvxopt import matrix, spmatrix
-from cvxopt.solvers import options, qp
+import cvxopt
+from cvxopt.solvers import qp
 from numpy import array, ndarray
+from scipy.sparse import csc_matrix
 
 from .conversions import linear_from_box_inequalities
-from .typing import CvxoptReadyMatrix
+
+cvxopt.solvers.options["show_progress"] = False  # disable verbose output
 
 
-options["show_progress"] = False  # disable cvxopt output
-
-
-def cvxopt_matrix(M: CvxoptReadyMatrix) -> matrix:
+def to_cvxopt(
+    M: Union[ndarray, csc_matrix]
+) -> Union[cvxopt.matrix, cvxopt.spmatrix]:
     """
     Convert matrix to CVXOPT format.
 
@@ -48,27 +49,30 @@ def cvxopt_matrix(M: CvxoptReadyMatrix) -> matrix:
         Matrix in CVXOPT format.
     """
     if isinstance(M, ndarray):
-        return matrix(M)
-    if isinstance(M, (spmatrix, matrix)):
-        return M
+        return cvxopt.matrix(M)
     coo = M.tocoo()
-    return spmatrix(
+    return cvxopt.spmatrix(
         coo.data.tolist(), coo.row.tolist(), coo.col.tolist(), size=M.shape
     )
 
 
 def cvxopt_solve_qp(
-    P: CvxoptReadyMatrix,
-    q: CvxoptReadyMatrix,
-    G: Optional[CvxoptReadyMatrix] = None,
-    h: Optional[CvxoptReadyMatrix] = None,
-    A: Optional[CvxoptReadyMatrix] = None,
-    b: Optional[CvxoptReadyMatrix] = None,
-    lb: Optional[CvxoptReadyMatrix] = None,
-    ub: Optional[CvxoptReadyMatrix] = None,
+    P: Union[ndarray, csc_matrix],
+    q: ndarray,
+    G: Optional[Union[ndarray, csc_matrix]] = None,
+    h: Optional[ndarray] = None,
+    A: Optional[Union[ndarray, csc_matrix]] = None,
+    b: Optional[ndarray] = None,
+    lb: Optional[ndarray] = None,
+    ub: Optional[ndarray] = None,
     solver: str = None,
     initvals: Optional[ndarray] = None,
     verbose: bool = False,
+    maxiters: Optional[int] = None,
+    abstol: Optional[float] = None,
+    reltol: Optional[float] = None,
+    feastol: Optional[float] = None,
+    refinement: Optional[int] = None,
 ) -> Optional[ndarray]:
     """
     Solve a Quadratic Program defined as:
@@ -117,6 +121,22 @@ def cvxopt_solve_qp(
         Warm-start guess vector.
     verbose :
         Set to `True` to print out extra information.
+    maxiters :
+        Maximum number of iterations.
+    abstol :
+        Absolute tolerance on the duality gap. See [Vandenberghe2010]_ or
+        *e.g.* [tolprimer]_ for a primer on the duality gap.
+    reltol :
+        Relative tolerance on the duality gap. See [Vandenberghe2010]_ or
+        *e.g.* [tolprimer]_ for a primer on the duality gap.
+    feastol :
+        Tolerance for feasibility conditions, that is, for the primal residual
+        equality and inequality constraints. See *e.g.* [tolprimer]_ for a
+        primer on primal-dual residuals.
+    refinement :
+        Number of iterative refinement steps when solving KKT equations
+        (default: ``0`` if the problem has no second-order cone or matrix
+        inequality constraints; ``1`` otherwise).
 
     Returns
     -------
@@ -146,19 +166,45 @@ def cvxopt_solve_qp(
     -----
     CVXOPT only considers the lower entries of `P`, therefore it will use a
     different cost than the one intended if a non-symmetric matrix is provided.
+
+    See the `Algorithm Parameters
+    <https://cvxopt.org/userguide/coneprog.html#algorithm-parameters>`_ section
+    of the solver documentation for more details and default values of the
+    solver parameters.
     """
     if lb is not None or ub is not None:
         G, h = linear_from_box_inequalities(G, h, lb, ub)
-    options["show_progress"] = verbose
-    args = [cvxopt_matrix(P), cvxopt_matrix(q)]
+
+    # Update solver options if applicable
+    options: Dict[str, Any] = {"show_progress": verbose}
+    if maxiters:
+        options["maxiters"] = maxiters
+    if abstol:
+        options["abstol"] = abstol
+    if reltol:
+        options["reltol"] = reltol
+    if feastol:
+        options["feastol"] = feastol
+    if refinement:
+        options["refinement"] = refinement
+
+    args = [to_cvxopt(P), to_cvxopt(q)]
     kwargs = {"G": None, "h": None, "A": None, "b": None}
     if G is not None and h is not None:
-        kwargs["G"] = cvxopt_matrix(G)
-        kwargs["h"] = cvxopt_matrix(h)
+        kwargs["G"] = to_cvxopt(G)
+        kwargs["h"] = to_cvxopt(h)
     if A is not None and b is not None:
-        kwargs["A"] = cvxopt_matrix(A)
-        kwargs["b"] = cvxopt_matrix(b)
-    sol = qp(*args, solver=solver, initvals=initvals, **kwargs)
+        kwargs["A"] = to_cvxopt(A)
+        kwargs["b"] = to_cvxopt(b)
+    initvals_dict: Optional[Dict[str, cvxopt.matrix]] = None
+    if initvals is not None:
+        initvals_dict = {"x": to_cvxopt(initvals)}
+
+    original_options = cvxopt.solvers.options
+    cvxopt.solvers.options = options
+    sol = qp(*args, solver=solver, initvals=initvals_dict, **kwargs)
+    cvxopt.solvers.options = original_options
+
     if "optimal" not in sol["status"]:
         return None
     return array(sol["x"]).reshape((q.shape[0],))
