@@ -28,34 +28,32 @@ does not make any rank assumption contrary to :ref:`CVXOPT <CVXOPT rank
 assumptions>` or :ref:`qpSWIFT <qpSWIFT rank assumptions>`).
 """
 
-from typing import Optional
+from typing import Optional, Union
 from warnings import warn
 
 import osqp
-
+import scipy.sparse as spa
 from numpy import hstack, inf, ndarray, ones
 from osqp import OSQP
-from scipy import sparse
+from scipy.sparse import csc_matrix
 
-from .conversions import linear_from_box_inequalities
-from .typing import DenseOrCSCMatrix
-from .typing import warn_about_sparse_conversion
+from .conversions import (
+    linear_from_box_inequalities,
+    warn_about_sparse_conversion,
+)
 
 
 def osqp_solve_qp(
-    P: DenseOrCSCMatrix,
-    q: DenseOrCSCMatrix,
-    G: Optional[DenseOrCSCMatrix] = None,
-    h: Optional[DenseOrCSCMatrix] = None,
-    A: Optional[DenseOrCSCMatrix] = None,
-    b: Optional[DenseOrCSCMatrix] = None,
-    lb: Optional[DenseOrCSCMatrix] = None,
-    ub: Optional[DenseOrCSCMatrix] = None,
-    initvals: Optional[DenseOrCSCMatrix] = None,
+    P: Union[ndarray, csc_matrix],
+    q: ndarray,
+    G: Optional[Union[ndarray, csc_matrix]] = None,
+    h: Optional[ndarray] = None,
+    A: Optional[Union[ndarray, csc_matrix]] = None,
+    b: Optional[ndarray] = None,
+    lb: Optional[ndarray] = None,
+    ub: Optional[ndarray] = None,
+    initvals: Optional[ndarray] = None,
     verbose: bool = False,
-    eps_abs: float = 1e-4,
-    eps_rel: float = 1e-4,
-    polish: bool = True,
     **kwargs,
 ) -> Optional[ndarray]:
     """
@@ -95,16 +93,6 @@ def osqp_solve_qp(
         Warm-start guess vector.
     verbose :
         Set to `True` to print out extra information.
-    eps_abs :
-        Absolute convergence tolerance of the solver. Lower values yield more
-        precise solutions at the cost of computation time.
-    eps_rel :
-        Relative convergence tolerance of the solver. Lower values yield more
-        precise solutions at the cost of computation time.
-    polish :
-        Perform `polishing <https://osqp.org/docs/solver/#polishing>`_, an
-        additional step where the solver tries to improve the accuracy of the
-        solution. Default is ``True``.
 
     Returns
     -------
@@ -129,38 +117,56 @@ def osqp_solve_qp(
 
     Notes
     -----
-    As of OSQP 0.6.1, the default values for both absolute and relative
-    tolerances are set to ``1e-3``, which results in low solver times but
-    imprecise solutions compared to the other QP solvers. We lower them to
-    ``1e-5`` so that OSQP behaves closer to the other solvers in terms of
-    numerical accuracy.
+    Keyword arguments are forwarded to OSQP. For instance, we can call
+    ``osqp_solve_qp(P, q, G, h, u, eps_abs=1e-8, eps_rel=0.0)``. OSQP settings
+    include the following:
 
-    All other keyword arguments are forwarded to the OSQP solver. For instance,
-    you can call ``osqp_solve_qp(P, q, G, h, u, alpha=1.42)``. See the `solver
-    documentation <https://osqp.org/docs/interfaces/solver_settings.html>`_ for
-    details.
+    .. list-table::
+       :widths: 30 70
+       :header-rows: 1
+
+       * - Name
+         - Description
+       * - ``max_iter``
+         - Maximum number of iterations.
+       * - ``time_limit``
+         - Run time limit in seconds, 0 to disable.
+       * - ``eps_abs``
+         - Absolute feasibility tolerance. See `Convergence
+           <https://osqp.org/docs/solver/index.html#convergence>`__.
+       * - ``eps_rel``
+         - Relative feasibility tolerance. See `Convergence
+           <https://osqp.org/docs/solver/index.html#convergence>`__.
+       * - ``eps_prim_inf``
+         - Primal infeasibility tolerance.
+       * - ``eps_dual_inf``
+         - Dual infeasibility tolerance.
+       * - ``polish``
+         - Perform polishing. See `Polishing
+           <https://osqp.org/docs/solver/#polishing>`_.
+
+    Check out the `OSQP settings
+    <https://osqp.org/docs/interfaces/solver_settings.html>`_ documentation for
+    all available settings..
+
+    Lower values for absolute or relative tolerances yield more precise
+    solutions at the cost of computation time. See *e.g.* [tolprimer]_ for an
+    overview of solver tolerances.
     """
     if isinstance(P, ndarray):
         warn_about_sparse_conversion("P")
-        P = sparse.csc_matrix(P)
+        P = csc_matrix(P)
     if lb is not None or ub is not None:
         G, h = linear_from_box_inequalities(G, h, lb, ub)
     solver = OSQP()
-    kwargs.update(
-        {
-            "eps_abs": eps_abs,
-            "eps_rel": eps_rel,
-            "polish": polish,
-            "verbose": verbose,
-        }
-    )
+    kwargs["verbose"] = verbose
     if A is not None and b is not None:
         if isinstance(A, ndarray):
             warn_about_sparse_conversion("A")
-            A = sparse.csc_matrix(A)
+            A = csc_matrix(A)
         if G is not None and h is not None:
             l_inf = -inf * ones(len(h))
-            qp_A = sparse.vstack([G, A], format="csc")
+            qp_A = spa.vstack([G, A], format="csc")
             qp_l = hstack([l_inf, b])
             qp_u = hstack([h, b])
             solver.setup(P=P, q=q, A=qp_A, l=qp_l, u=qp_u, **kwargs)
@@ -169,7 +175,7 @@ def osqp_solve_qp(
     elif G is not None and h is not None:
         if isinstance(G, ndarray):
             warn_about_sparse_conversion("G")
-            G = sparse.csc_matrix(G)
+            G = csc_matrix(G)
         l_inf = -inf * ones(len(h))
         solver.setup(P=P, q=q, A=G, l=l_inf, u=h, **kwargs)
     else:  # no inequality nor equality constraint
