@@ -130,7 +130,11 @@ def qpswift_solve_qp(
 
     Notes
     -----
-    The option dictionary accepts the following settings:
+    All other keyword arguments are forwarded as options to the qpSWIFT solver.
+    For instance, you can call ``qpswift_solve_qp(P, q, G, h, ABSTOL=1e-5)``.
+    See the solver documentation for details.
+
+    For a quick overview, the solver accepts the following settings:
 
     .. list-table::
        :widths: 30 70
@@ -139,13 +143,17 @@ def qpswift_solve_qp(
        * - Name
          - Effect
        * - MAXITER
-         - maximum number of iterations needed
+         - Maximum number of iterations needed.
        * - ABSTOL
-         - absolute tolerance
+         - Absolute tolerance on the duality gap. See *e.g.* [tolprimer]_ for a
+           primer on the duality gap and solver tolerances.
        * - RELTOL
-         - relative tolerance
+         - Relative tolerance on the residuals :math:`r_x = P x + G^T z + q`
+           (dual residual), :math:`r_y = A x - b` (primal residual on equality
+           constraints) and :math:`r_z = h - G x - s` (primal residual on
+           inequality constraints). See equation (21) in [Pandala2019]_.
        * - SIGMA
-         - maximum centering allowed
+         - Maximum centering allowed.
 
     If a verbose output shows that the maximum number of iterations is reached,
     check e.g. (1) the rank of your equality constraint matrix and (2) that
@@ -155,15 +163,9 @@ def qpswift_solve_qp(
     little more care than the other solvers. For instance, make sure you don't
     have zero rows in your input matrices, as it can `make the solver
     numerically unstable <https://github.com/qpSWIFT/qpSWIFT/issues/3>`_.
-
-    Notes
-    -----
-    All other keyword arguments are forwarded as options to the qpSWIFT solver.
-    For instance, you can call ``qpswift_solve_qp(P, q, G, h, ABSTOL=1e-5)``.
-    See the solver documentation for details.
     """
     if initvals is not None:
-        print("qpSWIFT: note that warm-start values ignored by wrapper")
+        print("qpSWIFT: warm-start values are ignored by wrapper")
     if lb is not None or ub is not None:
         G, h = linear_from_box_inequalities(G, h, lb, ub)
     result: dict = {}
@@ -179,9 +181,9 @@ def qpswift_solve_qp(
         else:  # no equality constraint
             result = qpSWIFT.run(q, h, P, G, opts=kwargs)
     else:  # no inequality constraint
-        # See https://qpswift.github.io/index.html#updates
+        # See https://github.com/qpSWIFT/qpSWIFT/issues/2
         raise NotImplementedError(
-            "QP without inequality constraints is still WIP for qpSWIFT"
+            "QPs without inequality constraints are not handled by qpSWIFT"
         )
     exit_flag = result["basicInfo"]["ExitFlag"]
     if exit_flag != 0:
