@@ -20,15 +20,18 @@
 
 """Solver interface for SCS"""
 
-from typing import Any, Dict, Optional
+from typing import Any, Dict, Optional, Union
 from warnings import warn
 
 import numpy as np
+import scipy.sparse as spa
+from numpy import ndarray
 from numpy.linalg import norm
-from scipy import sparse
+from scipy.sparse import csc_matrix
+from scipy.sparse.linalg import lsqr
 from scs import solve
 
-from .typing import DenseOrCSCMatrix, warn_about_sparse_conversion
+from .conversions import warn_about_sparse_conversion
 
 # See https://www.cvxgrp.org/scs/api/exit_flags.html#exit-flags
 __status_val_meaning__ = {
@@ -45,21 +48,91 @@ __status_val_meaning__ = {
 }
 
 
+def __add_box_cone(
+    n: int,
+    lb: Optional[ndarray],
+    ub: Optional[ndarray],
+    cone: Dict[str, Any],
+    data: Dict[str, Any],
+) -> None:
+    """
+    Add box cone to the problem.
+
+    Parameters
+    ----------
+    n :
+        Number of optimization variables.
+    lb :
+        Lower bound constraint vector.
+    ub :
+        Upper bound constraint vector.
+    cone :
+        SCS cone dictionary.
+    data :
+        SCS data dictionary.
+
+    Notes
+    -----
+    See the `SCS Cones <https://www.cvxgrp.org/scs/api/cones.html>`__
+    documentation for details.
+    """
+    cone["bl"] = lb if lb is not None else np.full((n,), -np.inf)
+    cone["bu"] = ub if ub is not None else np.full((n,), +np.inf)
+    zero_row = csc_matrix((1, n))
+    data["A"] = spa.vstack(
+        ((data["A"],) if "A" in data else ()) + (zero_row, -spa.eye(n)),
+        format="csc",
+    )
+    data["b"] = np.hstack(
+        ((data["b"],) if "b" in data else ()) + (1.0, np.zeros(n))
+    )
+
+
+def __solve_unconstrained(
+    P: Union[ndarray, csc_matrix], q: ndarray
+) -> ndarray:
+    """
+    Solve unconstrained QP, warning if the problem is unbounded.
+
+    Parameters
+    ----------
+    P :
+        Primal quadratic cost matrix.
+    q :
+        Primal quadratic cost vector.
+
+    Returns
+    -------
+    :
+        Solution to the QP, if found, otherwise ``None``.
+
+    Raises
+    ------
+    ValueError
+        If the quadratic program is not unbounded below.
+    """
+    x = lsqr(P, -q)[0]
+    if norm(P @ x + q) > 1e-9:
+        raise ValueError(
+            "problem is unbounded below, "
+            "q has component in the nullspace of P"
+        )
+    return x
+
+
 def scs_solve_qp(
-    P: DenseOrCSCMatrix,
-    q: np.ndarray,
-    G: Optional[DenseOrCSCMatrix] = None,
-    h: Optional[np.ndarray] = None,
-    A: Optional[DenseOrCSCMatrix] = None,
-    b: Optional[np.ndarray] = None,
-    lb: Optional[np.ndarray] = None,
-    ub: Optional[np.ndarray] = None,
-    initvals: Optional[np.ndarray] = None,
-    eps_abs: float = 1e-7,
-    eps_rel: float = 1e-7,
+    P: Union[ndarray, csc_matrix],
+    q: ndarray,
+    G: Optional[Union[ndarray, csc_matrix]] = None,
+    h: Optional[ndarray] = None,
+    A: Optional[Union[ndarray, csc_matrix]] = None,
+    b: Optional[ndarray] = None,
+    lb: Optional[ndarray] = None,
+    ub: Optional[ndarray] = None,
+    initvals: Optional[ndarray] = None,
     verbose: bool = False,
     **kwargs,
-) -> Optional[np.ndarray]:
+) -> Optional[ndarray]:
     """
     Solve a Quadratic Program defined as:
 
@@ -96,12 +169,6 @@ def scs_solve_qp(
         Upper bound constraint vector.
     initvals :
         Warm-start guess vector (not used).
-    eps_abs : float
-        Absolute feasibility tolerance, see `Termination criteria
-        <https://www.cvxgrp.org/scs/algorithm/index.html#termination>`_.
-    eps_rel : float
-        Relative feasibility tolerance, see `Termination criteria
-        <https://www.cvxgrp.org/scs/algorithm/index.html#termination>`_.
     verbose :
         Set to `True` to print out extra information.
 
@@ -110,41 +177,62 @@ def scs_solve_qp(
     :
         Solution to the QP, if found, otherwise ``None``.
 
+    Raises
+    ------
+    ValueError
+        If the quadratic program is not unbounded below.
+
     Notes
     -----
-    Keyword arguments are forwarded as is to SCS. For instance, you can call
-    ``scs_solve_qp(P, q, G, h, normalize=True)``. Solver settings for SCS are
-    described `here <https://www.cvxgrp.org/scs/api/settings.html#settings>`_.
+    Keyword arguments are forwarded as is to SCS. For instance, we can call
+    ``scs_solve_qp(P, q, G, h, eps_abs=1e-6, eps_rel=1e-4)``. SCS settings
+    include the following:
 
-    As of SCS 3.0.1, the default feasibility tolerances are set ``1e-4``,
-    resulting in larger inequality constraint violations than with other
-    solvers on the README and unit test problems. We lower them to ``1e-9`` so
-    that SCS behaves closer to the other solvers. If you don't need that much
-    precision, increase them for better performance.
+    .. list-table::
+       :widths: 30 70
+       :header-rows: 1
+
+       * - Name
+         - Description
+       * - ``max_iters``
+         - Maximum number of iterations to run.
+       * - ``time_limit_secs``
+         - Time limit for solve run in seconds (can be fractional). 0 is
+           interpreted as no limit.
+       * - ``eps_abs``
+         - Absolute feasibility tolerance. See `Termination criteria
+           <https://www.cvxgrp.org/scs/algorithm/index.html#termination>`__.
+       * - ``eps_rel``
+         - Relative feasibility tolerance. See `Termination criteria
+           <https://www.cvxgrp.org/scs/algorithm/index.html#termination>`__.
+       * - ``eps_infeas``
+         - Infeasibility tolerance (primal and dual), see `Certificate of
+           infeasibility
+           <https://www.cvxgrp.org/scs/algorithm/index.html#certificate-of-infeasibility>`_.
+       * - ``normalize``
+         - Whether to perform heuristic data rescaling. See `Data equilibration
+           <https://www.cvxgrp.org/scs/algorithm/equilibration.html#equilibration>`__.
+
+    Check out the `SCS settings
+    <https://www.cvxgrp.org/scs/api/settings.html#settings>`_ documentation for
+    all available settings.
     """
-    if isinstance(P, np.ndarray):
+    if isinstance(P, ndarray):
         warn_about_sparse_conversion("P")
-        P = sparse.csc_matrix(P)
-    if isinstance(G, np.ndarray):
+        P = csc_matrix(P)
+    if isinstance(G, ndarray):
         warn_about_sparse_conversion("G")
-        G = sparse.csc_matrix(G)
-    if isinstance(A, np.ndarray):
+        G = csc_matrix(G)
+    if isinstance(A, ndarray):
         warn_about_sparse_conversion("A")
-        A = sparse.csc_matrix(A)
-    kwargs.update(
-        {
-            "eps_abs": eps_abs,
-            "eps_rel": eps_rel,
-            "verbose": verbose,
-        }
-    )
+        A = csc_matrix(A)
     data: Dict[str, Any] = {"P": P, "c": q}
     cone: Dict[str, Any] = {}
     if initvals is not None:
         data["x"] = initvals
     if A is not None and b is not None:
         if G is not None and h is not None:
-            data["A"] = sparse.vstack([A, G], format="csc")
+            data["A"] = spa.vstack([A, G], format="csc")
             data["b"] = np.hstack([b, h])
             cone["z"] = b.shape[0]  # zero cone
             cone["l"] = h.shape[0]  # positive cone
@@ -156,24 +244,12 @@ def scs_solve_qp(
         data["A"] = G
         data["b"] = h
         cone["l"] = h.shape[0]  # positive cone
-    else:  # no constraint
-        x = sparse.linalg.lsqr(P, -q)[0]
-        if norm(P @ x + q) > 1e-9:
-            raise ValueError(
-                "problem is unbounded below, "
-                "q has component in the nullspace of P"
-            )
-        return x
+    elif lb is None and ub is None:  # no constraint
+        return __solve_unconstrained(P, q)
     if lb is not None or ub is not None:
         n = P.shape[1]
-        cone["bl"] = lb if lb is not None else np.full((n,), -np.inf)
-        cone["bu"] = ub if ub is not None else np.full((n,), +np.inf)
-        zero_row = sparse.csc_matrix((1, n))
-        data["A"] = sparse.vstack(
-            (data["A"], zero_row, -sparse.eye(n)),
-            format="csc",
-        )
-        data["b"] = np.hstack((data["b"], 1.0, np.zeros(n)))
+        __add_box_cone(n, lb, ub, cone, data)
+    kwargs["verbose"] = verbose
     solution = solve(data, cone, **kwargs)
     status_val = solution["info"]["status_val"]
     if status_val != 1:
