@@ -26,13 +26,17 @@ import unittest
 import warnings
 
 import scipy
-
-from numpy import allclose, array, dot, ones, random
+from numpy import array, dot, ones, random
 from numpy.linalg import norm
 from scipy.sparse import csc_matrix
 
-from qpsolvers import available_solvers, sparse_solvers
-from qpsolvers import solve_qp, solve_safer_qp
+from qpsolvers import (
+    available_solvers,
+    dense_solvers,
+    solve_qp,
+    solve_safer_qp,
+    sparse_solvers,
+)
 from qpsolvers.exceptions import NoSolverSelected, SolverNotFound
 
 # Raising a ValueError when the problem is unbounded below is desired but not
@@ -45,6 +49,17 @@ behavior_on_unbounded = {
 }
 
 
+def solve_qp_with_test_params(*args, **kwargs):
+    """
+    Call ``solve_qp`` with additional solver parameters
+    """
+    params = {}
+    if kwargs["solver"] == "proxqp":
+        params["eps_abs"] = 1e-9
+    kwargs.update(params)
+    return solve_qp(*args, **kwargs)
+
+
 class TestSolveQP(unittest.TestCase):
 
     """
@@ -123,7 +138,7 @@ class TestSolveQP(unittest.TestCase):
         """
         P, q, G, h, A, b = self.get_dense_problem()
         with self.assertRaises(NoSolverSelected):
-            solve_qp(P, q, G, h, A, b, solver=None)
+            solve_qp_with_test_params(P, q, G, h, A, b, solver=None)
 
     def test_solver_not_found(self):
         """
@@ -131,7 +146,7 @@ class TestSolveQP(unittest.TestCase):
         """
         P, q, G, h, A, b = self.get_dense_problem()
         with self.assertRaises(SolverNotFound):
-            solve_qp(P, q, G, h, A, b, solver="ideal")
+            solve_qp_with_test_params(P, q, G, h, A, b, solver="ideal")
 
     @staticmethod
     def get_test(solver):
@@ -151,17 +166,29 @@ class TestSolveQP(unittest.TestCase):
 
         def test(self):
             P, q, G, h, A, b = self.get_dense_problem()
-            x = solve_qp(P, q, G, h, A, b, solver=solver)
-            x_sp = solve_qp(P, q, G, h, A, b, solver=solver, sym_proj=True)
+            x = solve_qp_with_test_params(P, q, G, h, A, b, solver=solver)
+            x_sp = solve_qp_with_test_params(
+                P, q, G, h, A, b, solver=solver, sym_proj=True
+            )
             self.assertIsNotNone(x)
             self.assertIsNotNone(x_sp)
             known_solution = array([0.30769231, -0.69230769, 1.38461538])
-            sol_tolerance = 1e-4 if solver == "ecos" else 1e-8
-            ineq_tolerance = 1e-10
+            sol_tolerance = (
+                2e-4
+                if solver == "osqp"
+                else 5e-4
+                if solver == "scs"
+                else 1e-4
+                if solver == "ecos"
+                else 1e-8
+            )
+            eq_tolerance = 1e-10
+            ineq_tolerance = 2e-4 if solver == "scs" else 1e-10
             self.assertLess(norm(x - known_solution), sol_tolerance)
             self.assertLess(norm(x_sp - known_solution), sol_tolerance)
             self.assertLess(max(dot(G, x) - h), ineq_tolerance)
-            self.assertTrue(allclose(dot(A, x), b))
+            self.assertLess(max(dot(A, x) - b), eq_tolerance)
+            self.assertLess(min(dot(A, x) - b), eq_tolerance)
 
         return test
 
@@ -180,6 +207,10 @@ class TestSolveQP(unittest.TestCase):
         -------
         test : function
             Test function for that solver.
+
+        Note
+        ----
+        This function relies on "quadprog" to find groundtruth solutions.
         """
 
         def test(self):
@@ -188,15 +219,18 @@ class TestSolveQP(unittest.TestCase):
             A = array([[1.0, 0.0, 0.0], [0.0, 0.4, 0.5]])
             b = array([-0.5, -1.2])
             b0 = array([b[0]])
-            lb = array([-1.0, -1.0, -1.0])
+            lb = array([-0.5, -2, -0.8])
             ub = array([+1.0, +1.0, +1.0])
 
             cases = [
                 {"P": P, "q": q},
                 {"P": P, "q": q, "G": G, "h": h},
-                {"P": P, "q": q, "A": A, "b": b},
                 {"P": P, "q": q, "G": G[0], "h": h0},
+                {"P": P, "q": q, "A": A, "b": b},
                 {"P": P, "q": q, "A": A[0], "b": b0},
+                {"P": P, "q": q, "lb": lb, "ub": None},
+                {"P": P, "q": q, "lb": None, "ub": ub},
+                {"P": P, "q": q, "lb": lb, "ub": ub},
                 {"P": P, "q": q, "G": G, "h": h, "A": A, "b": b},
                 {"P": P, "q": q, "G": G[0], "h": h0, "A": A, "b": b},
                 {"P": P, "q": q, "G": G, "h": h, "A": A[0], "b": b0},
@@ -204,40 +238,89 @@ class TestSolveQP(unittest.TestCase):
                 {
                     "P": P,
                     "q": q,
-                    "G": G[0],
-                    "h": h0,
-                    "A": A[0],
-                    "b": b0,
+                    "G": G,
+                    "h": h,
+                    "A": A,
+                    "b": b,
                     "lb": lb,
                     "ub": None,
                 },
                 {
                     "P": P,
                     "q": q,
-                    "G": G[0],
-                    "h": h0,
-                    "A": A[0],
-                    "b": b0,
+                    "G": G,
+                    "h": h,
+                    "A": A,
+                    "b": b,
                     "lb": None,
                     "ub": ub,
                 },
                 {
                     "P": P,
                     "q": q,
-                    "G": G[0],
-                    "h": h0,
-                    "A": A[0],
-                    "b": b0,
+                    "G": G,
+                    "h": h,
+                    "A": A,
+                    "b": b,
+                    "lb": lb,
+                    "ub": ub,
+                },
+                {
+                    "P": P,
+                    "q": q,
+                    "G": G,
+                    "h": h,
+                    "lb": lb,
+                    "ub": None,
+                },
+                {
+                    "P": P,
+                    "q": q,
+                    "G": G,
+                    "h": h,
+                    "lb": None,
+                    "ub": ub,
+                },
+                {
+                    "P": P,
+                    "q": q,
+                    "G": G,
+                    "h": h,
                     "lb": lb,
                     "ub": ub,
                 },
             ]
 
-            for (i, case) in enumerate(cases):
-                quadprog_solution = solve_qp(solver="quadprog", **case)
-                for solver in available_solvers:
-                    x = solve_qp(solver=solver, **case)
-                    self.assertLess(norm(x - quadprog_solution), 2e-4)
+            for (i, test_case) in enumerate(cases):
+                if "G" not in test_case and solver == "qpswift":
+                    # QPs without inequality constraints not handled by qpSWIFT
+                    continue
+                test_comp = {
+                    k: v.shape if v is not None else "None"
+                    for k, v in test_case.items()
+                }
+                quadprog_solution = solve_qp_with_test_params(
+                    solver="quadprog", **test_case
+                )
+                self.assertIsNotNone(
+                    quadprog_solution,
+                    f"Baseline failed on parameters: {test_comp}",
+                )
+                solver_solution = solve_qp_with_test_params(
+                    solver=solver, **test_case
+                )
+                sol_tolerance = (
+                    2e-3
+                    if solver == "osqp"
+                    else 5e-4
+                    if solver == "ecos"
+                    else 2e-4
+                )
+                self.assertLess(
+                    norm(solver_solution - quadprog_solution),
+                    sol_tolerance,
+                    f"Solver failed on parameters: {test_comp}",
+                )
 
         return test
 
@@ -261,14 +344,26 @@ class TestSolveQP(unittest.TestCase):
             P, q, G, h, A, b = self.get_dense_problem()
             lb = array([-1.0, -2.0, -0.5])
             ub = array([1.0, -0.2, 1.0])
-            x = solve_qp(P, q, G, h, A, b, lb, ub, solver=solver)
+            x = solve_qp_with_test_params(
+                P, q, G, h, A, b, lb, ub, solver=solver
+            )
             self.assertIsNotNone(x)
             known_solution = array([0.41463415, -0.41463415, 1.0])
-            sol_tolerance = 1e-6 if solver in ["ecos", "scs"] else 1e-8
+            sol_tolerance = (
+                5e-3
+                if solver == "osqp"
+                else 5e-5
+                if solver == "scs"
+                else 1e-6
+                if solver == "ecos"
+                else 1e-8
+            )
+            eq_tolerance = 1e-10
             ineq_tolerance = 1e-10
             self.assertLess(norm(x - known_solution), sol_tolerance)
             self.assertLess(max(dot(G, x) - h), ineq_tolerance)
-            self.assertTrue(allclose(dot(A, x), b))
+            self.assertLess(max(dot(A, x) - b), eq_tolerance)
+            self.assertLess(min(dot(A, x) - b), eq_tolerance)
 
         return test
 
@@ -291,10 +386,16 @@ class TestSolveQP(unittest.TestCase):
 
         def test(self):
             P, q, G, h, A, b = self.get_dense_problem()
-            x = solve_qp(P, q, solver=solver)
+            x = solve_qp_with_test_params(P, q, solver=solver)
             self.assertIsNotNone(x)
             known_solution = array([-0.64705882, -1.17647059, -1.82352941])
-            sol_tolerance = 1e-3 if solver == "ecos" else 1e-6
+            sol_tolerance = (
+                1e-3
+                if solver == "ecos"
+                else 1e-5
+                if solver == "osqp"
+                else 1e-6
+            )
             self.assertLess(norm(x - known_solution), sol_tolerance)
 
         return test
@@ -318,11 +419,23 @@ class TestSolveQP(unittest.TestCase):
 
         def test(self):
             P, q, G, h, A, b = self.get_dense_problem()
-            x = solve_qp(P, q, G, h, solver=solver)
+            x = solve_qp_with_test_params(P, q, G, h, solver=solver)
             self.assertIsNotNone(x)
             known_solution = array([-0.49025721, -1.57755261, -0.66484801])
-            sol_tolerance = 1e-3 if solver == "ecos" else 1e-6
-            ineq_tolerance = 1e-7 if solver == "scs" else 1e-10
+            sol_tolerance = (
+                1e-3
+                if solver == "ecos"
+                else 2e-6
+                if solver == "osqp"
+                else 1e-6
+            )
+            ineq_tolerance = (
+                2e-6
+                if solver == "osqp"
+                else 1e-7
+                if solver == "scs"
+                else 1e-10
+            )
             self.assertLess(norm(x - known_solution), sol_tolerance)
             self.assertLess(max(dot(G, x) - h), ineq_tolerance)
 
@@ -347,12 +460,22 @@ class TestSolveQP(unittest.TestCase):
 
         def test(self):
             P, q, G, h, A, b = self.get_dense_problem()
-            x = solve_qp(P, q, A=A, b=b, solver=solver)
+            x = solve_qp_with_test_params(P, q, A=A, b=b, solver=solver)
             self.assertIsNotNone(x)
             known_solution = array([0.28026906, -1.55156951, 2.27130045])
-            sol_tolerance = 1e-5 if solver in ["ecos", "scs"] else 1e-8
+            sol_tolerance = (
+                5e-4
+                if solver == "osqp"
+                else 1e-5
+                if solver in ["ecos", "scs"]
+                else 1e-6
+                if solver == "highs"
+                else 1e-8
+            )
+            eq_tolerance = 1e-9
             self.assertLess(norm(x - known_solution), sol_tolerance)
-            self.assertTrue(allclose(dot(A, x), b))
+            self.assertLess(max(dot(A, x) - b), eq_tolerance)
+            self.assertLess(min(dot(A, x) - b), eq_tolerance)
 
         return test
 
@@ -376,20 +499,32 @@ class TestSolveQP(unittest.TestCase):
         def test(self):
             P, q, G, h, A, b = self.get_dense_problem()
             G, h = G[1], h[1].reshape((1,))
-            x = solve_qp(P, q, G, h, A, b, solver=solver)
+            x = solve_qp_with_test_params(P, q, G, h, A, b, solver=solver)
             self.assertIsNotNone(x)
             known_solution = array([0.30769231, -0.69230769, 1.38461538])
             sol_tolerance = (
-                1e-5
+                5e-4
+                if solver == "osqp"
+                else 1e-5
                 if solver == "scs"
                 else 1e-6
                 if solver in ["cvxopt", "ecos"]
+                else 5e-8
+                if solver == "qpswift"
+                else 1e-8
+            )
+            eq_tolerance = 5e-10 if solver in ["osqp", "scs"] else 1e-10
+            ineq_tolerance = (
+                1e-7
+                if solver == "scs"
+                else 2e-8
+                if solver == "qpswift"
                 else 1e-8
             )
-            ineq_tolerance = 1e-7 if solver == "scs" else 1e-8
             self.assertLess(norm(x - known_solution), sol_tolerance)
-            self.assertLess(dot(G, x) - h, ineq_tolerance)
-            self.assertTrue(allclose(dot(A, x), b))
+            self.assertLess(max(dot(G, x) - h), ineq_tolerance)
+            self.assertLess(max(dot(A, x) - b), eq_tolerance)
+            self.assertLess(min(dot(A, x) - b), eq_tolerance)
 
         return test
 
@@ -411,14 +546,20 @@ class TestSolveQP(unittest.TestCase):
 
         def test(self):
             P, q, G, h, _, _ = self.get_dense_problem()
-            if solver in sparse_solvers:
+            if solver not in dense_solvers:
                 with self.assertRaises(NotImplementedError):
                     solve_safer_qp(P, q, G, h, sr=1e-4, solver=solver)
                 return
             x = solve_safer_qp(P, q, G, h, sr=1e-4, solver=solver)
             self.assertIsNotNone(x)
             known_solution = array([-0.49021915, -1.57749935, -0.66477954])
-            sol_tolerance = 1e-4 if solver in ["ecos", "scs"] else 1e-6
+            sol_tolerance = (
+                2e-3
+                if solver == "proxqp"  # test params not applied
+                else 1e-4
+                if solver in ["ecos", "scs"]
+                else 1e-6
+            )
             ineq_tolerance = 1e-7 if solver == "scs" else 1e-10
             self.assertLess(norm(x - known_solution), sol_tolerance)
             self.assertLess(max(dot(G, x) - h), ineq_tolerance)
@@ -444,11 +585,25 @@ class TestSolveQP(unittest.TestCase):
 
         def test(self):
             P, q, G, h = self.get_sparse_problem()
-            x = solve_qp(P, q, G, h, solver=solver)
+            x = solve_qp_with_test_params(P, q, G, h, solver=solver)
             self.assertIsNotNone(x)
             known_solution = array([2.0] * 149 + [3.0])
-            sol_tolerance = 1e-3 if solver == "gurobi" else 1e-7
-            ineq_tolerance = 1e-7
+            sol_tolerance = (
+                5e-3
+                if solver == "cvxopt"
+                else 1e-3
+                if solver == "gurobi"
+                else 5e-4
+                if solver == "osqp"
+                else 1e-4
+                if solver == "scs"
+                else 1e-6
+                if solver == "highs"
+                else 1e-7
+            )
+            ineq_tolerance = (
+                1e-4 if solver == "scs" else 5e-7 if solver == "osqp" else 1e-7
+            )
             self.assertLess(norm(x - known_solution), sol_tolerance)
             self.assertLess(max(G * x - h), ineq_tolerance)
 
@@ -475,14 +630,18 @@ class TestSolveQP(unittest.TestCase):
             P, q, G, h = self.get_sparse_problem()
             lb = +2.2 * ones(q.shape)
             ub = +2.4 * ones(q.shape)
-            x = solve_qp(P, q, G, h, lb=lb, ub=ub, solver=solver)
+            x = solve_qp_with_test_params(
+                P, q, G, h, lb=lb, ub=ub, solver=solver
+            )
             self.assertIsNotNone(x)
             known_solution = array([2.2] * 149 + [2.4])
             sol_tolerance = (
                 1e-3
                 if solver == "gurobi"
+                else 5e-5
+                if solver == "osqp"
                 else 1e-7
-                if solver == "scs"
+                if solver in ["cvxopt", "scs"]
                 else 1e-8
             )
             ineq_tolerance = 1e-10
@@ -512,7 +671,13 @@ class TestSolveQP(unittest.TestCase):
             P, q, G, h = self.get_sparse_problem()
             lb = +0.5 * ones(q.shape)
             ub = +1.5 * ones(q.shape)
-            x = solve_qp(P, q, G, h, lb=lb, ub=ub, solver=solver)
+            if solver == "cvxopt":
+                # Skipping this test for CVXOPT for now
+                # See https://github.com/cvxopt/cvxopt/issues/229
+                return
+            x = solve_qp_with_test_params(
+                P, q, G, h, lb=lb, ub=ub, solver=solver
+            )
             self.assertIsNone(x)
 
         return test
@@ -537,7 +702,7 @@ class TestSolveQP(unittest.TestCase):
             P, q, G, h, A, b = self.get_dense_problem()
             known_solution = array([0.30769231, -0.69230769, 1.38461538])
             initvals = known_solution + 0.1 * random.random(3)
-            x = solve_qp(
+            x = solve_qp_with_test_params(
                 P,
                 q,
                 G,
@@ -549,11 +714,21 @@ class TestSolveQP(unittest.TestCase):
                 verbose=True,  # increases coverage
             )
             self.assertIsNotNone(x)
-            sol_tolerance = 1e-4 if solver == "ecos" else 1e-8
-            ineq_tolerance = 1e-10
+            sol_tolerance = (
+                5e-4
+                if solver == "scs"
+                else 2e-4
+                if solver == "osqp"
+                else 1e-4
+                if solver == "ecos"
+                else 1e-8
+            )
+            eq_tolerance = 5e-5 if solver == "osqp" else 1e-10
+            ineq_tolerance = 2e-4 if solver in ["osqp", "scs"] else 1e-10
             self.assertLess(norm(x - known_solution), sol_tolerance)
             self.assertLess(max(dot(G, x) - h), ineq_tolerance)
-            self.assertTrue(allclose(dot(A, x), b))
+            self.assertLess(max(dot(A, x) - b), eq_tolerance)
+            self.assertLess(min(dot(A, x) - b), eq_tolerance)
 
         return test
 
@@ -587,7 +762,7 @@ class TestSolveQP(unittest.TestCase):
             q = array([-1.0, -2, 0, 3e-4])
             # q is in the nullspace of P, so the problem is unbounded below
             with self.assertRaises(ValueError):
-                solve_qp(P, q, solver=solver)
+                solve_qp_with_test_params(P, q, solver=solver)
 
         return test
 
@@ -605,21 +780,25 @@ for solver in available_solvers:
         f"test_bounds_{solver}",
         TestSolveQP.get_test_bounds(solver),
     )
-    setattr(
-        TestSolveQP,
-        f"test_no_cons_{solver}",
-        TestSolveQP.get_test_no_cons(solver),
-    )
+    if solver != "qpswift":
+        # QPs without inequality constraints are not handled by qpSWIFT
+        setattr(
+            TestSolveQP,
+            f"test_no_cons_{solver}",
+            TestSolveQP.get_test_no_cons(solver),
+        )
     setattr(
         TestSolveQP,
         f"test_no_eq_{solver}",
         TestSolveQP.get_test_no_eq(solver),
     )
-    setattr(
-        TestSolveQP,
-        f"test_no_ineq_{solver}",
-        TestSolveQP.get_test_no_ineq(solver),
-    )
+    if solver != "qpswift":
+        # QPs without inequality constraints are not handled by qpSWIFT
+        setattr(
+            TestSolveQP,
+            f"test_no_ineq_{solver}",
+            TestSolveQP.get_test_no_ineq(solver),
+        )
     setattr(
         TestSolveQP,
         f"test_one_ineq_{solver}",
@@ -657,7 +836,3 @@ for solver in available_solvers:
             f"test_raise_on_unbounded_below_{solver}",
             TestSolveQP.get_test_raise_on_unbounded_below(solver),
         )
-
-
-if __name__ == "__main__":
-    unittest.main()
